/*

  Graphic user's interface library
  Folder widget

  Author(s): Ilja V. Ryndin <support@rusbitech.ru>

  Start date: *

*/

#ifndef FLYFM_FOLDERWIDGET_H
#define FLYFM_FOLDERWIDGET_H

#include <flyfm/gui_export.h>

#include <QAbstractItemView>
#include <QElapsedTimer>

#include <flyfm/model.h>
#include <flyfm/imanageiconsize.h>

class QGridLayout;

namespace FlyFM
{
class FOpId;
class Navigation;
class FileActionsProvider;
class IconSizeActionsProvider;
class Job;
class ItemViewTouchGesturesManager;
class ViewSignalsObject;

class FLYFM_GUI_EXPORT FolderWidget : public QWidget, public IManageIconSize
{
    Q_OBJECT
public:
    explicit FolderWidget(QWidget *parent = nullptr);
    explicit FolderWidget(ModelFilter::EntryFilters, QWidget *parent = nullptr);
    virtual ~FolderWidget();

    Model *model();
    const Model *model() const;

    void refresh();

    Navigation *navigation();

    Entry currentEntry();
    void setCurrent(const Entry &entry);

    QGridLayout *layout();

    void setHideOpenActions(bool enable);
    void setHideFindAction(bool enable);
    void setHideSendActions(bool enable);
    void setHideViewChooseActions(bool enable);
    void setHideShowHiddenAction(bool enable);

    static QByteArray defaultState(bool tabletMode);
    QByteArray saveState() const;
    void restoreState(const QByteArray &state);

    IconSize iconSize() const override;

    void setSelectionMode(QAbstractItemView::SelectionMode mode);

    EntryList selectedEntries() const;
    Entry highlightedEntry() const;

    void setFindMode(bool enable);
    bool isInFindMode() const
    {
        return m_findMode;
    }

    bool showHiddenEntries() const;
    void setShowHiddenEntries(bool enable);
    void setAlternatingRowColorsInTable(bool on);

    bool showDotDot();
    void setShowDotDot(bool on);

    void setActive(bool active);
    bool isActive() const;

    void setShowLinkEmblem(bool on = true);
    bool showLinkEmblem() const;

    void setShowPropertyEmblems(bool on = true);
    bool showPropertyEmblems() const;

    void setShowSelectionControl(bool on);
    bool showSelectionControl() const;

    void setFileActionsUpdatesEnabled(bool enabled);

    FileActionsProvider *fileActionsProvider();
    QList<QAction *> fileActions() const;

    QAbstractItemView *itemView() const;
    int itemCount() const;
    int selectedItemCount() const;

public slots:
    void setIconSize(IconSize size) override;

    void setEntry(const Entry &);

    void applyTableView();
    void applyIconView();
    void applyListView();

    void highlightEntry(const Entry &);
    void selectEntries(const EntryList &entries);
    void sort(int, Qt::SortOrder);

signals:
    void viewModeChanged();
    void itemSelectionChanged();
    void entryHighlighted(const Entry &);
    void openActionRequested(const EntryList &, const Entry &currentEntry);
    void iconSizeChanged(int);
    void iconSizeChanged(IconSize);
    void currentChanged(const Entry &);
    void activated();
    void itemViewChanged(QAbstractItemView *);
    void dirListed(const Entry &entry);
    void itemCountChanged(int items);
    void selectedItemCountChanged(int items);

protected slots:
    void updatingStarted();
    void updatingFinished();

    void slotCurrentChanged(const Entry &);
    void dropPopupContextMenu(const QPoint &, Qt::DropActions, const EntryList &, const Entry &);
    void arkExtractContextMenu(const QPoint &, const QString &dbusService, const QString &dbusPath, const Entry &dst);
    void dropTitledShortcutMenu(const QPoint &pos, const QString &url, const QString &title, const Entry &dst);
    void customPopupContextMenu(const QPoint &, const QModelIndexList &);
    void showTableColumnsMenu(const QPoint &);
    void openAction(const QModelIndexList &indexes);
    void activatedAction(const QModelIndex &);

    void increaseIconSizeAction();
    void decreaseIconSizeAction();
    void updateModelPresentation();

    void updateShortcutFileAction();
    void updateFileActions();

    void updateSmbEntryJobFinished(Job *job);
    void updateNetworkPlaceJobFinished(Job *job);

    void emitSelectedItemCountChanged();

protected:
    bool eventFilter(QObject *, QEvent *) override;
    void changeEvent(QEvent *event) override;

    void changeViewTo(Model::Mode);
    void setView(QAbstractItemView *view);

    QList<Model::PropertyInfo> availablePropertyNames() const;
    QList<Model::PropertyInfo> availableSortableProperties() const;

    bool isPropertyHidden(int propertyIdx) const;
    void setPropertyHidden(int propertyIdx, bool hidden);
    int sortPropertyIndex() const;
    Qt::SortOrder sortPropertyOrder() const;

    void highlightEntry(const Entry &, QAbstractItemView::ScrollHint hint);

    QMap<Model::Mode, IconSize> m_iconSizeMap;

    mutable QByteArray m_tableHeaderState;
    mutable QList<int> m_tableHiddenSections;

    QList<int> m_hiddenSections;

    Navigation *m_navigation;
    Model *m_model;
    QAbstractItemView *m_view;

    bool hideOpenActions;
    bool hideSendActions;
    bool hideFindAction;
    bool hideViewChooseActions;
    bool hideShowHiddenAction;

    QAbstractItemView::SelectionMode m_selectionMode;

    bool m_findMode;

    QGridLayout *m_layout;

    bool m_isActive;
    bool m_activateItemOnSingleClick;
    ItemViewTouchGesturesManager *m_touchGesturesManager;

    IconSizeActionsProvider *m_iconSizeActionsProvider;
    FileActionsProvider *m_fileActionsProvider;

    bool m_fileActionsUpdatesEnabled;

    QElapsedTimer m_singleClickElapsedTimer;

    friend class ColumnMenu;
    friend class SortMenu;
    friend class PopupMenu;

private:
    void localSaveTableHeaderState() const;
    void localRestoreTableHeaderState();

    void setActivateItemOnSingleClick(bool on);

    void setHiddenProperties(const QStringList &props);
    void applyHiddenProperties();

    void updatePalette();

    bool m_showSelectionControl;
    bool m_alternatingRowColorsInTable;

    ViewSignalsObject *m_viewSignalsObject;
};

}

#endif // FLYFM_FOLDERWIDGET_H
