/*

  Graphic user's interface library
  File system model

  Author(s): Ilja V. Ryndin <support@rusbitech.ru>

  Start date: *

*/

#ifndef FLYFM_MODEL_H
#define FLYFM_MODEL_H

#include <flyfm/core_export.h>

#include <QAbstractItemModel>

#include <flyfm/modelfilter.h>

namespace FlyFM
{

class Entry;
class EntryList;

class ModelPrivate;

class FLYFM_CORE_EXPORT Model : public QAbstractItemModel
{
    Q_OBJECT
    friend class ModelItem;
public:
    enum Mode { UnspecMode = 0, DesktopMode = 1, IconMode = 2, TreeMode = 4, TableMode = 8, ListMode = 16, PlacesMode = 32 };
    enum SortOrder { UnspecOrder = 0, AscendingOrder = 1, DescendingOrder = 2 };

    explicit Model(Mode mode = DesktopMode, ModelFilter::EntryFilters entryFilters = ModelFilter::AllEntries, QObject *parent = nullptr);
    ~Model();

    void refresh();
    void updatePresentation();

    void rebuild();
    void rebuild(const Entry &rootEntry);
    void rebuildWithMode(Mode mode);
    void rebuildWithEntryFilters(ModelFilter::EntryFilters entryFilters);
    void rebuildWithPropertySortIndex(int propertySortIndex, SortOrder sortOrder = AscendingOrder);

    bool showHiddenEntries() const;
    void setShowHiddenEntries(bool enable);

    bool showDotDot();
    void setShowDotDot(bool on);

    ModelFilter &filter();
    const ModelFilter &filter() const;
    void flushFilter();

    Entry actualRootEntry();

    bool canFetchMore(const QModelIndex &index) const override;
    void fetchMore(const QModelIndex &index) override;

    int columnCount(const QModelIndex &parent = QModelIndex()) const override;
    QVariant data(const QModelIndex &index, int role = Qt::DisplayRole) const override;
    QModelIndex index(int row, int column, const QModelIndex &parent = QModelIndex()) const override;

    QModelIndex parent(const QModelIndex &index) const override;
    QVariant headerData(int section, Qt::Orientation orientation, int role = Qt::DisplayRole) const override;
    int rowCount(const QModelIndex &parent = QModelIndex()) const override;
    bool hasChildren(const QModelIndex &parent = QModelIndex()) const override;
    Qt::ItemFlags flags(const QModelIndex &index) const override;
    Qt::DropActions supportedDropActions() const override;
    bool dropMimeData(const QMimeData *data, Qt::DropAction action, int row, int column, const QModelIndex &parent) override;
    QMimeData *mimeData(const QModelIndexList &indexes) const override;
    QStringList mimeTypes() const override;
    void sort(int column, Qt::SortOrder order) override;

    virtual QList<QModelIndex> pathIndexList(const Entry &entry);
    virtual QModelIndex indexOfEntry(const Entry &entry);

    // void setPropertyEnabled( const QString &, bool );
    void setNameFilters(const QStringList &);
    void setNameFiltersForFilesOnly(bool on = true);

    Mode mode() const;
    SortOrder sortOrder() const;
    int propertySortIndex() const;
    struct PropertyInfo
    {
        QString name;
        int index;
    };
    QList<PropertyInfo> sortableProperties() const;
    bool isNumericSortingEnabled() const;
    bool isLocaleSortingEnabled() const;

    QStringList propertyNames() const;
    QStringList internalPropertyNames() const;

    ModelFilter::EntryFilters entryFilters() const;
    QStringList nameFilters() const;

    Entry entryByIndex(const QModelIndex &index) const;
    EntryList indexesToEntries(const QModelIndexList &) const;

    void setShowLinkEmblem(bool on = true);
    bool showLinkEmblem() const;

    void setShowPropertyEmblems(bool on = true);
    bool showPropertyEmblems() const;

    void setArchivesAreFiles(bool on = true);

    static bool isHiddenChild(const Entry &);

signals:
    void updatingStarted();
    void updatingFinished();
    void updated(const QModelIndex &);
    void dirListed(const Entry &entry);

protected:
    QScopedPointer<ModelPrivate> d_ptr;
    Model(ModelPrivate &dd, Mode mode = DesktopMode, ModelFilter::EntryFilters entryFilters = ModelFilter::AllEntries, QObject *parent = nullptr);

    void beginInsertRows(const QModelIndex &parent, int first, int last);
    void endInsertRows();

private:
    Q_DECLARE_PRIVATE(Model)
};

}

FLYFM_CORE_EXPORT QDataStream &operator<< (QDataStream &out, const FlyFM::Model::Mode &mode);
FLYFM_CORE_EXPORT QDataStream &operator>> (QDataStream &in, FlyFM::Model::Mode &mode);

#endif // FLYFM_MODEL_H
