#ifndef SCHEDULEDUPDATEVIEW_H
#define SCHEDULEDUPDATEVIEW_H

#include "progressicon.h"

#include <QObject>
#include <QPointer>
#include <QMap>
#include <QQueue>
#include <QModelIndex>
#include <QPainter>
#include <QTimerEvent>

#include <functional>

class QTimer;
class QShowEvent;
class QResizeEvent;
class QPaintEvent;
class QAbstractItemView;

namespace FlyFM
{

class ScheduledViewUpdater;
class ViewSignalsObject;

class ScheduledUpdateViewCommon
{
    friend class ScheduledViewUpdater;
protected:
    virtual ~ScheduledUpdateViewCommon();

    virtual void setUpdater(QObject *context, std::function<void()> updater);
    virtual QModelIndex callIndexBelow(const QModelIndex &index) const;
    virtual void callDataChanged(const QModelIndex &topLeft, const QModelIndex &bottomRight,
                                 const QVector<int> &roles = QVector<int>()) = 0;
};

template <class T>
class ScheduledUpdateView : public T, public ScheduledUpdateViewCommon
{
public:
    ScheduledUpdateView(ViewSignalsObject *signalsObject, QWidget *parent = nullptr);

    void scheduleUpdating();

    void setModel(QAbstractItemModel *model) override;
protected:
    void scrollContentsBy(int dx, int dy) override;
    void showEvent(QShowEvent *) override;
    void resizeEvent(QResizeEvent *event) override;
    void paintEvent(QPaintEvent *event) override;
    void timerEvent(QTimerEvent *event) override;

    void rowsAboutToBeRemoved(const QModelIndex &parent, int start, int end) override;
    void rowsInserted(const QModelIndex &parent, int start, int end) override;
    void dataChanged(const QModelIndex &topLeft, const QModelIndex &bottomRight,
                     const QVector<int> &roles = QVector<int>()) override;

private:
    ScheduledViewUpdater *m_viewUpdater;
    ProgressIcon m_progressIcon;
    int m_progressTimerId;
};

class ScheduledViewUpdater : public QObject
{
    Q_OBJECT
public:
    ScheduledViewUpdater(QAbstractItemView *view);
    ~ScheduledViewUpdater();

    void scheduleUpdating();
    void setModel(QAbstractItemModel *model);

private slots:
    void updateVisibleIndexes();
    void updatePendingIndexes();

    void layoutChanged();
private:
    QTimer *m_updateTimer;
    QQueue<QPersistentModelIndex> pendingIndexes;
    QPointer<QAbstractItemView> m_view;
    ScheduledUpdateViewCommon *m_viewCommon;
};

template <class T>
ScheduledUpdateView<T>::ScheduledUpdateView(ViewSignalsObject *signalsObject, QWidget *parent)
    : T(signalsObject, parent)
    , m_progressTimerId(0)
{
    m_viewUpdater = new ScheduledViewUpdater(this);
}

template <class T>
void ScheduledUpdateView<T>::scheduleUpdating()
{
    m_viewUpdater->scheduleUpdating();
}

template <class T>
void ScheduledUpdateView<T>::setModel(QAbstractItemModel *newModel)
{
    m_viewUpdater->setModel(newModel);

    T::setModel(newModel);
}

template <class T>
void ScheduledUpdateView<T>::scrollContentsBy(int dx, int dy)
{
    T::scrollContentsBy(dx, dy);
    scheduleUpdating();
}

template <class T>
void ScheduledUpdateView<T>::showEvent(QShowEvent *e)
{
    T::showEvent(e);
    scheduleUpdating();
}

template <class T>
void ScheduledUpdateView<T>::resizeEvent(QResizeEvent *event)
{
    T::resizeEvent(event);
    scheduleUpdating();
}

template<class T>
void ScheduledUpdateView<T>::paintEvent(QPaintEvent *event)
{
    if (T::model()->data(T::rootIndex(), Qt::UserRole + 2).toBool()) {
        if (m_progressTimerId == 0) {
            m_progressTimerId = T::startTimer(100);
            return;
        }
        QPainter painter(T::viewport());
        m_progressIcon.paint(&painter, QRect(16, 16, 128, 128));
    } else {
        if (m_progressTimerId != 0) {
            T::killTimer(m_progressTimerId);
            m_progressTimerId = 0;
        }
        T::paintEvent(event);
    }
}

template<class T>
void ScheduledUpdateView<T>::timerEvent(QTimerEvent *event)
{
    if (event->timerId() == m_progressTimerId) {
        T::viewport()->update();
    } else {
        T::timerEvent(event);
    }
}

template <class T>
void ScheduledUpdateView<T>::rowsAboutToBeRemoved(const QModelIndex &parent, int start, int end)
{
    T::rowsAboutToBeRemoved(parent, start, end);
    scheduleUpdating();
}

template <class T>
void ScheduledUpdateView<T>::rowsInserted(const QModelIndex &parent, int start, int end)
{
    T::rowsInserted(parent, start, end);
    scheduleUpdating();
}

template<class T>
void ScheduledUpdateView<T>::dataChanged(const QModelIndex &topLeft, const QModelIndex &bottomRight, const QVector<int> &roles)
{
    if (roles.contains(Qt::UserRole + 2) && topLeft == bottomRight && topLeft == T::currentIndex()) {
        const auto area = T::viewport()->rect();
        const auto r = T::visualRect(topLeft);
        const bool isVisible = r.isValid() && area.top() <= r.top() && r.bottom() <= area.bottom();
        if (!isVisible) {
            T::scrollTo(topLeft, T::PositionAtTop);
        }

    }
    T::dataChanged(topLeft, bottomRight, roles);
}

}

#endif // SCHEDULEDUPDATEVIEW_H
