#ifndef FLYFM_VFSPLACEBASE_H
#define FLYFM_VFSPLACEBASE_H

#include <flyfm/vfs_export.h>

#include <QString>
#include <QMap>
#include <QMutex>
#include <QSharedPointer>
#include <QPointer>

#include <functional>

#include "entrylist.h"

namespace FlyFM
{

class FLYFM_VFS_EXPORT VFSPlaceBase
{
    friend class VFS;
    friend class VFSPrivate;

public:
    ~VFSPlaceBase();

    Entry createPlace(const QString &place, int type, const QString &dirPath = QString());
    Entry createPlace(const Entry &entry);
    Entry createPendingPlace(const QString &place, int type, QObject *obj, std::function<void(const QString &)> fn);
    bool destroyPlace(const Entry &placeEntry);
    void setPlaceLocalPath(const Entry &placeEntry, const QString &localPath = QString());
    bool hasPlace(const Entry &placeEntry) const;
    bool hasPlace(const QString &placePath) const;

    bool addEntryToPlace(const Entry &entry, const Entry &placeEntry);
    EntryList addEntriesToPlace(const EntryList &entries, const Entry &placeEntry);
    bool removeEntryFromPlace(const Entry &entry, const Entry &placeEntry);
    bool placeHasChildEntry(const Entry &placeEntry, const Entry &childEntry) const;
    void clearPlace(const Entry &placeEntry);

    const Entry placeEntry(const QString &path) const;
    Entry dirPlaceEntry(const QString &localPath) const;
    int placeOrderIndex(const Entry &placeEntry, const Entry &parentPlaceEntry);
    EntryList childEntries(const Entry &parent) const;
    bool hasChilds(const Entry &parent) const;
    bool isPathContainsPlace(const QString &path) const;
    QStringList exploreParentPlacesHistory(const QString &path, const QString &root) const;

private:
    VFSPlaceBase();
    VFSPlaceBase(const VFSPlaceBase &);
    VFSPlaceBase &operator= (const VFSPlaceBase &);

    Entry checkAndGetParentPlace(const Entry &entry, const Entry &root) const;

    QString placeRelatedPath(const QString &path) const;
    void createIfPendingPlace(const QString &parentPath) const;

    struct PlaceTreeNode {
        PlaceTreeNode(const Entry &e) : entry(e) {}

        Entry entry;
        QList<QWeakPointer<PlaceTreeNode>> parents;
        QList<QSharedPointer<PlaceTreeNode>> children;
    };

    QStringList _exploreParentPlacesHistory(const QString &place, QSharedPointer<PlaceTreeNode> rootNode) const;

    mutable QMutex m_mutex;

    using PlaceNodeMap = QMap<QString, QSharedPointer<PlaceTreeNode>>;
    PlaceNodeMap m_placeNodeMap;

    struct PendingPlace {
        PendingPlace() {}
        PendingPlace(const QString &p, int t, QObject *o, std::function<void(const QString &)> fn)
            : place(p), type(t), obj(o), createFn(std::move(fn)) {}

        QString place;
        int type;
        QPointer<QObject> obj;
        std::function<void(const QString &)> createFn;
    };

    using PendingPlacesMap = QMap<QString, PendingPlace>;
    mutable PendingPlacesMap m_pendingPlacesMap;
};
}

#endif
